from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("challenge", "0023_auto_20151002_0948"),
    ]

    operations = [
        migrations.AlterField(
            model_name="qualification",
            name="name",
            field=models.CharField(max_length=255, verbose_name="Nom de la classe"),
        ),
    ]



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("user", "0004_userprofile_formation"),
    ]

    operations = [
        migrations.AddField(
            model_name="userprofile",
            name="social_security",
            field=models.CharField(blank=True, max_length=16),
        ),
    ]



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("challenge", "0012_auto_20150903_1723"),
    ]

    operations = [
        migrations.AddField(
            model_name="qualification",
            name="n_bikes",
            field=models.PositiveSmallIntegerField(
                null=True, verbose_name="Nombre de vélos"
            ),
        ),
        migrations.AddField(
            model_name="qualification",
            name="n_helmets",
            field=models.PositiveSmallIntegerField(
                null=True, verbose_name="Nombre de casques"
            ),
        ),
        migrations.AddField(
            model_name="qualification",
            name="n_participants",
            field=models.PositiveSmallIntegerField(
                null=True, verbose_name="Nombre de participants"
            ),
        ),
    ]



# Generated by Django 1.11.4 on 2017-08-30 13:14
from __future__ import unicode_literals

from django.db import migrations, models
from memoize import delete_memoized

from defivelo.roles import user_cantons


def vsow_to_hw(apps, schema_editor):
    UserManagedState = apps.get_model("user", "UserManagedState")
    for ums in UserManagedState.objects.all():
        if ums.canton == "VS-OW":
            ums.canton = "WS"
            ums.save()

    UserProfile = apps.get_model("user", "UserProfile")
    for up in UserProfile.objects.all():
        cantons = ["WS" if c == "VS-OW" else c for c in up.activity_cantons]
        up.activity_cantons = cantons
        if up.affiliation_canton == "VS-OW":
            up.affiliation_canton = "WS"
        up.save()
    delete_memoized(user_cantons)


def hw_to_vsow(apps, schema_editor):
    UserManagedState = apps.get_model("user", "UserManagedState")
    for ums in UserManagedState.objects.all():
        if ums.canton == "WS":
            ums.canton = "VS-OW"
            ums.save()

    UserProfile = apps.get_model("user", "UserProfile")
    for up in UserProfile.objects.all():
        cantons = ["VS-OW" if c == "WS" else c for c in up.activity_cantons]
        up.activity_cantons = cantons
        if up.affiliation_canton == "WS":
            up.affiliation_canton = "VS-OW"
        up.save()
    delete_memoized(user_cantons)


class Migration(migrations.Migration):
    dependencies = [
        ("user", "0041_auto_20170824_1212"),
    ]

    operations = [
        migrations.RunPython(vsow_to_hw, hw_to_vsow),
        migrations.AlterField(
            model_name="usermanagedstate",
            name="canton",
            field=models.CharField(
                choices=[
                    ("AR", "Appenzell Ausserrhoden"),
                    ("BS", "Basel-Stadt"),
                    ("BE", "Berne"),
                    ("FR", "Fribourg"),
                    ("GE", "Geneva"),
                    ("JU", "Jura"),
                    ("LU", "Lucerne"),
                    ("NE", "Neuchatel"),
                    ("SG", "St. Gallen"),
                    ("VS", "Valais"),
                    ("VD", "Vaud"),
                    ("ZH", "Zurich"),
                    ("WS", "Haut-Valais"),
                ],
                max_length=5,
                verbose_name="Canton",
            ),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="activity_cantons",
            field=models.CharField(
                blank=True,
                choices=[
                    ("AR", "Appenzell Ausserrhoden"),
                    ("BS", "Basel-Stadt"),
                    ("BE", "Berne"),
                    ("FR", "Fribourg"),
                    ("GE", "Geneva"),
                    ("JU", "Jura"),
                    ("LU", "Lucerne"),
                    ("NE", "Neuchatel"),
                    ("SG", "St. Gallen"),
                    ("VS", "Valais"),
                    ("VD", "Vaud"),
                    ("ZH", "Zurich"),
                    ("WS", "Haut-Valais"),
                ],
                max_length=38,
                verbose_name="Défi Vélo mobile",
            ),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="affiliation_canton",
            field=models.CharField(
                blank=True,
                choices=[
                    ("", "---------"),
                    ("AR", "Appenzell Ausserrhoden"),
                    ("BS", "Basel-Stadt"),
                    ("BE", "Berne"),
                    ("FR", "Fribourg"),
                    ("GE", "Geneva"),
                    ("JU", "Jura"),
                    ("LU", "Lucerne"),
                    ("NE", "Neuchatel"),
                    ("SG", "St. Gallen"),
                    ("VS", "Valais"),
                    ("VD", "Vaud"),
                    ("ZH", "Zurich"),
                    ("WS", "Haut-Valais"),
                ],
                max_length=5,
                verbose_name="Canton d'affiliation",
            ),
        ),
    ]



from __future__ import unicode_literals

from django.db import migrations, models


def takefirstcanton(apps, schema_editor):
    # We get the model from the versioned app registry;
    # if we directly import it, it'll be the wrong version
    try:
        UserProfile = apps.get_model("user", "UserProfile")
    except Exception:
        return
    db_alias = schema_editor.connection.alias
    for profile in UserProfile.objects.using(db_alias).all():
        if profile.activity_cantons:
            profile.activity_cantons = profile.activity_cantons[0]
            profile.save()


class Migration(migrations.Migration):
    dependencies = [
        ("user", "0016_auto_20160913_1155"),
    ]

    operations = [
        migrations.RunPython(takefirstcanton),
        migrations.AlterField(
            model_name="userprofile",
            name="activity_cantons",
            field=models.CharField(
                choices=[
                    ("BS", "Basel-Stadt"),
                    ("BE", "Berne"),
                    ("FR", "Fribourg"),
                    ("GE", "Geneva"),
                    ("LU", "Lucerne"),
                    ("NE", "Neuchatel"),
                    ("SG", "St. Gallen"),
                    ("VS", "Valais"),
                    ("VD", "Vaud"),
                    ("ZH", "Zurich"),
                ],
                verbose_name="Canton d'affiliation",
                max_length=2,
            ),
        ),
    ]



# Generated by Django 2.2.12 on 2020-05-18 12:28

import django.db.models.deletion
import parler.fields
import parler.models
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("salary", "0008_monthlycantonalvalidation"),
    ]

    operations = [
        migrations.CreateModel(
            name="MonthlyCantonalValidationUrl",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("url", models.URLField(verbose_name="URL")),
            ],
            options={
                "verbose_name": "URL pour validation mensuelle",
                "verbose_name_plural": "URLs pour validation mensuelle",
            },
            bases=(parler.models.TranslatableModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name="MonthlyCantonalValidationUrlTranslation",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "language_code",
                    models.CharField(
                        db_index=True, max_length=15, verbose_name="Language"
                    ),
                ),
                ("name", models.CharField(max_length=255, verbose_name="Nom")),
                (
                    "master",
                    parler.fields.TranslationsForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="translations",
                        to="salary.MonthlyCantonalValidationUrl",
                    ),
                ),
            ],
            options={
                "verbose_name": "URL pour validation mensuelle Translation",
                "db_table": "salary_monthlycantonalvalidationurl_translation",
                "db_tablespace": "",
                "managed": True,
                "default_permissions": (),
                "unique_together": {("language_code", "master")},
            },
            bases=(parler.models.TranslatedFieldsModelMixin, models.Model),
        ),
    ]



# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2016, 2020 Didier Raboud <me+defivelo@odyx.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import datetime

from django.test import TestCase
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from bs4 import BeautifulSoup

from apps.common import (
    DV_SEASON_AUTUMN,
    DV_SEASON_LAST_SPRING_MONTH,
    DV_SEASON_SPRING,
    DV_SEASON_STATE_ARCHIVED,
    DV_SEASON_STATE_OPEN,
    DV_SEASON_STATE_PLANNING,
    DV_SEASON_STATE_RUNNING,
    DV_SEASON_STATES,
    DV_STATES,
)
from apps.common.forms import SWISS_DATE_INPUT_FORMAT
from apps.orga.tests.factories import OrganizationFactory
from apps.user import FORMATION_M1, FORMATION_M2
from apps.user.tests.factories import UserFactory
from defivelo.tests.utils import (
    AuthClient,
    CoordinatorAuthClient,
    PowerUserAuthClient,
    StateManagerAuthClient,
)

from .. import (
    AVAILABILITY_FIELDKEY,
    CHOSEN_AS_LEADER,
)
from ..models import HelperSessionAvailability
from .factories import QualificationFactory, SeasonFactory, SessionFactory

freeforallurls = ["season-list"]
restrictedgenericurls = ["season-create"]
restrictedspecificurls = [
    "season-detail",
    "season-update",
    "season-helperlist",
    "season-actorlist",
    "season-availabilities",
    "season-staff-update",
    "season-set-running",
    "season-set-open",
]
restrictedhelperspecificurls = [
    "season-availabilities-update",
    "season-planning",
]


class SeasonTestCaseMixin(TestCase):
    def setUp(self):
        self.users = [UserFactory() for i in range(3)]

        self.season = SeasonFactory()
        self.mycantons = [c.canton for c in self.client.user.managedstates.all()]
        if self.mycantons:
            self.season.cantons = self.mycantons
        else:
            self.season.cantons = [
                DV_STATES[0],
            ]
        self.season.save()

        self.sessions = []
        self.canton_orgas = []
        self.qualifs = []
        for canton in self.season.cantons:
            s = SessionFactory(orga__address_canton=canton)
            self.canton_orgas.append(s.orga)
            s.day = self.season.begin
            s.save()
            for i in range(0, 4):
                self.qualifs.append(QualificationFactory(session=s))
            self.sessions.append(s)

        self.foreigncantons = [c for c in DV_STATES if c not in self.mycantons]
        self.foreignseason = SeasonFactory(cantons=self.foreigncantons)

        self.foreignsessions = []
        for canton in self.foreigncantons:
            s = SessionFactory()
            s.orga.address_canton = canton
            s.orga.save()
            s.day = self.foreignseason.begin
            s.save()
            for i in range(0, 4):
                QualificationFactory(session=s)
            self.foreignsessions.append(s)


class AuthUserTest(SeasonTestCaseMixin):
    def setUp(self):
        self.client = AuthClient()
        super(AuthUserTest, self).setUp()

    def test_access_to_season_list(self):
        for symbolicurl in freeforallurls:
            url = reverse(symbolicurl)
            # Final URL is OK
            response = self.client.get(url, follow=True)
            self.assertEqual(response.status_code, 200, url)

    def test_no_access_to_season_create(self):
        for symbolicurl in restrictedgenericurls:
            url = reverse(symbolicurl)
            # Final URL is forbidden
            response = self.client.get(url, follow=True)
            self.assertEqual(response.status_code, 403, url)

    def test_no_access_to_season_detail(self):
        for symbolicurl in restrictedspecificurls:
            url = reverse(symbolicurl, kwargs={"pk": self.season.pk})
            # Final URL is forbidden
            response = self.client.get(url, follow=True)
            self.assertEqual(response.status_code, 403, url)
        for symbolicurl in restrictedhelperspecificurls:
            for helper in self.users:
                url = reverse(
                    symbolicurl, kwargs={"pk": self.season.pk, "helperpk": helper.pk}
                )
                # Final URL is forbidden
                response = self.client.get(url, follow=True)
                self.assertEqual(response.status_code, 403, url)

    def test_access_to_myseason_availabilities(self):
        urls = {
            symbolicurl: reverse(
                symbolicurl,
                kwargs={"pk": self.season.pk, "helperpk": self.client.user.pk},
            )
            for symbolicurl in restrictedhelperspecificurls
        }
        # Fails, we have no formation
        for k, url in urls.items():
            self.assertEqual(self.client.get(url).status_code, 403, url)

        # Fails, we have not a common canton
        self.client.user.profile.formation = FORMATION_M1
        self.client.user.profile.save()
        for k, url in urls.items():
            self.assertEqual(self.client.get(url).status_code, 403, url)

        # Works, we have it all
        self.client.user.profile.affiliation_canton = self.season.cantons[0]
        self.client.user.profile.save()
        response = self.client.get(urls["season-availabilities-update"])
        self.assertEqual(response.status_code, 200, ["season-availabilities-update"])
        self.assertTemplateUsed(response, "widgets/BSRadioSelect_option.html")

        # If, by mistake, the user got sent a url with a helperpk set to 0, it will redirect
        response = self.client.get(
            reverse(
                "season-availabilities-update",
                kwargs={"pk": self.season.pk, "helperpk": 0},
            )
        )
        self.assertEqual(
            response.status_code,
            302,
            "DEFIVELO-221: Availabilities' update URL with helperpk set to 0",
        )

        # The season is not open
        for state in DV_SEASON_STATES:
            self.season.state = state[0]
            self.season.save()
            response = self.client.get(urls["season-availabilities-update"])
            if state[0] == DV_SEASON_STATE_OPEN:
                self.assertEqual(
                    response.status_code, 200, urls["season-availabilities-update"]
                )
            elif state[0] == DV_SEASON_STATE_RUNNING:
                # When possible, we redirect to planning
                self.assertEqual(
                    response.status_code, 302, urls["season-availabilities-update"]
                )
            else:
                self.assertEqual(
                    response.status_code, 403, urls["season-availabilities-update"]
                )
        # Test the access to the planning
        for state in DV_SEASON_STATES:
            self.season.state = state[0]
            self.season.save()
            response = self.client.get(urls["season-planning"])
            if state[0] == DV_SEASON_STATE_RUNNING:
                # … We can see the season
                self.assertEqual(response.status_code, 200, urls["season-planning"])
                for exportformat in ["csv", "ods", "xls"]:
                    url = reverse(
                        "season-personal-planning-export",
                        kwargs={
                            "pk": self.season.pk,
                            "helperpk": self.client.user.pk,
                            "format": exportformat,
                        },
                    )
                    response = self.client.get(url, follow=True)
                    self.assertEqual(response.status_code, 200, url)
                url = reverse(
                    "season-personal-calendar",
                    kwargs={"pk": self.season.pk, "helperpk": self.client.user.pk},
                )
                response = self.client.get(url, follow=True)
                self.assertEqual(response.status_code, 200, url)
            elif state[0] == DV_SEASON_STATE_OPEN:
                # When possible, we redirect to availabilities
                self.assertEqual(response.status_code, 302, urls["season-planning"])
            else:
                self.assertEqual(response.status_code, 403, urls["season-planning"])

        self.season.state = DV_SEASON_STATE_OPEN
        self.season.save()

        # Fails, we have a common canton, but no Formation
        self.client.user.profile.formation = ""
        self.client.user.profile.save()
        for k, url in urls.items():
            self.assertEqual(self.client.get(url).status_code, 403, url)

    def test_access_to_sessions(self):
        # Make this user a M1 from this season's cantons
        self.client.user.profile.formation = FORMATION_M1
        self.client.user.profile.affiliation_canton = self.season.cantons[0]
        self.client.user.profile.save()

        # Test the access to the session on all season states
        for state in DV_SEASON_STATES:
            self.season.state = state[0]
            self.season.save()
            for session in self.sessions:
                session_kwargs = {
                    "seasonpk": self.season.pk,
                    "pk": session.pk,
                }
                weeklysessionlist = reverse(
                    "session-list",
                    kwargs={
                        "seasonpk": self.season.pk,
                        "year": session.day.year,
                        "week": session.day.strftime("%W"),
                    },
                )
                urls = [
                    weeklysessionlist,
                    reverse(
                        "session-create",
                        kwargs={
                            "seasonpk": self.season.pk,
                        },
                    ),
                    reverse(
                        "session-detail",
                        kwargs=session_kwargs,
                    ),
                    reverse(
                        "session-update",
                        kwargs=session_kwargs,
                    ),
                    reverse(
                        "session-staff-choices",
                        kwargs=session_kwargs,
                    ),
                    reverse(
                        "session-delete",
                        kwargs=session_kwargs,
                    ),
                ]
                for url in urls:
                    # Final URL is forbidden
                    response = self.client.get(url, follow=True)
                    self.assertEqual(response.status_code, 403, url)
                # Now add user to a Qualif of that Session
                q = session.qualifications.first()
                q.helpers.add(self.client.user)
                # Recheck all URLs
                for url in urls:
                    # Loop through the two cases
                    for visible in [True, False]:
                        session.visible = visible
                        session.save()
                        response = self.client.get(url, follow=True)
                        # When the season is running, we can access some stuff
                        if state[0] == DV_SEASON_STATE_RUNNING and (
                            url == weeklysessionlist
                            or (
                                visible
                                and url
                                in [
                                    reverse(
                                        "session-detail",
                                        kwargs=session_kwargs,
                                    ),
                                    reverse(
                                        "session-staff-choices",
                                        kwargs=session_kwargs,
                                    ),
                                ]
                            )
                        ):
                            self.assertEqual(response.status_code, 200, url)
                        else:
                            self.assertEqual(response.status_code, 403, url)
                # Drop it from the Qualif
                q.helpers.remove(self.client.user)


class StateManagerUserTest(SeasonTestCaseMixin):
    def setUp(self):
        self.client = StateManagerAuthClient()
        super(StateManagerUserTest, self).setUp()

    def test_access_to_season_list(self):
        for symbolicurl in restrictedgenericurls:
            url = reverse(symbolicurl)
            # Final URL is forbidden
            response = self.client.get(url, follow=True)
            self.assertEqual(response.status_code, 200, url)

    def test_access_to_season_detail(self):
        # The season is anything but archived
        for state in DV_SEASON_STATES:
            self.season.state = state[0]
            self.season.save()
            self.season.refresh_from_db()
            if state[0] != DV_SEASON_STATE_ARCHIVED:
                for symbolicurl in restrictedspecificurls:
                    url = reverse(symbolicurl, kwargs={"pk": self.season.pk})
                    # Final URL is OK
                    response = self.client.get(url, follow=True)
                    expected_status_code = 200
                    if (
                        symbolicurl == "season-set-running"
                        and not self.season.can_set_state_running
                    ):
                        # There are only some states in which we cna set the state to running
                        expected_status_code = 403
                    if (
                        symbolicurl == "season-set-open"
                        and not self.season.can_set_state_open
                    ):
                        # There are only some states in which we cna set the state to running
                        expected_status_code = 403
                    self.assertEqual(response.status_code, expected_status_code, url)

                for symbolicurl in restrictedhelperspecificurls:
                    for helper in self.users:
                        url = reverse(
                            symbolicurl,
                            kwargs={"pk": self.season.pk, "helperpk": helper.pk},
                        )
                        # Final URL is OK
                        response = self.client.get(url, follow=True)
                        self.assertEqual(response.status_code, 200, url)
                for exportformat in ["csv", "ods", "xls"]:
                    url = reverse(
                        "season-export",
                        kwargs={"pk": self.season.pk, "format": exportformat},
                    )
                    # Final URL is OK
                    response = self.client.get(url, follow=True)
                    self.assertEqual(response.status_code, 200, url)

    def test_season_runningplanning_accesses(self):
        # Loop over all states
        for state in DV_SEASON_STATES:
            self.season.state = state[0]
            self.season.save()
            url = reverse("season-set-running", kwargs={"pk": self.season.pk})
            response = self.client.get(url, follow=True)
            expected_status_code = 200
            if not self.season.can_set_state_running:
                expected_status_code = 403
            self.assertEqual(response.status_code, expected_status_code, url)

    def test_season_runningplanning(self):
        url = reverse("season-set-running", kwargs={"pk": self.season.pk})

        # Test that anything put that state works
        for state in DV_SEASON_STATES:
            # Set the season to be in preparation
            self.season.state = DV_SEASON_STATE_OPEN
            self.season.save()
            # Test that we can post to set it running
            response = self.client.post(url, {"state": state[0]})
            # 200 means that we're back on the page with an error
            expected_status_code = 200
            if state[0] == DV_SEASON_STATE_RUNNING:
                # 302 means that the post succeeded; so we can only post this state
                expected_status_code = 302
            self.assertEqual(response.status_code, expected_status_code, url)

    def test_season_openplanning_accesses(self):
        # Loop over all states
        for state in DV_SEASON_STATES:
            self.season.state = state[0]
            self.season.save()
            url = reverse("season-set-open", kwargs={"pk": self.season.pk})
            response = self.client.get(url, follow=True)
            expected_status_code = 200
            if not self.season.can_set_state_open:
                expected_status_code = 403
            self.assertEqual(response.status_code, expected_status_code, url)

    def test_season_openplanning(self):
        url = reverse("season-set-open", kwargs={"pk": self.season.pk})

        # Test that anything put that state works
        for state in DV_SEASON_STATES:
            # Set the season to be in preparation
            self.season.state = DV_SEASON_STATE_PLANNING
            self.season.save()
            # Test that we can post to set it running
            response = self.client.post(url, {"state": state[0]})
            # 200 means that we're back on the page with an error
            expected_status_code = 200
            if state[0] == DV_SEASON_STATE_OPEN:
                # 302 means that the post succeeded; so we can only post this state
                expected_status_code = 302
            self.assertEqual(response.status_code, expected_status_code, url)

    def test_season_creation(self):
        url = reverse("season-create")
        # Final URL is OK
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200, url)

        initial = {
            "year": 2015,
            "month_start": 1,
            "n_months": 6,
            "cantons": [],
            "leader": self.client.user.pk,
            "state": DV_SEASON_STATE_OPEN,
        }

        # 200 because we're back on the page, because cantons' empty
        response = self.client.post(url, initial)
        self.assertEqual(response.status_code, 200, url)

        initial["cantons"] = self.foreigncantons
        # 200 because we're back on the page, because cantons' not our cantons
        response = self.client.post(url, initial)
        self.assertEqual(response.status_code, 200, url)

        initial["cantons"] = self.mycantons
        # That works now
        response = self.client.post(url, initial)
        self.assertEqual(response.status_code, 302, url)

    def test_session_creation(self):
        # The season is anything but archived
        for state in DV_SEASON_STATES:
            if state[0] != DV_SEASON_STATE_ARCHIVED:
                self.season.state = state[0]
                self.season.save()

                url = reverse("session-create", kwargs={"seasonpk": self.season.pk})
                # Final URL is OK
                response = self.client.get(url, follow=True)
                self.assertEqual(response.status_code, 200, url)

                initial = {
                    "day": (self.season.begin).strftime(SWISS_DATE_INPUT_FORMAT),
                    "begin": "09:00",
                    "bikes_phone": "",
                }

                # 200 because we're back on the page, because orga' empty
                response = self.client.post(url, initial)
                self.assertEqual(response.status_code, 200, url)

                orga = OrganizationFactory(address_canton=self.foreignseason.cantons[0])
                initial["orga"] = orga.pk
                # 200 because we're back on the page, because orga is not
                # in our canton
                response = self.client.post(url, initial)
                self.assertEqual(response.status_code, 200, url)

                orga = OrganizationFactory(address_canton=self.season.cantons[0])
                initial["orga"] = orga.pk
                # That works now
                response = self.client.post(url, initial)
                self.assertEqual(response.status_code, 302, url)

    def test_user_can_edit_session(self):
        session = self.sessions[0]
        session.begin = datetime.time(9, 30)
        session.save()

        for state in DV_SEASON_STATES:
            if state[0] != DV_SEASON_STATE_ARCHIVED:
                self.season.state = state[0]
                self.season.save()
                self.client.post(
                    reverse(
                        "session-update",
                        kwargs={"pk": session.pk, "seasonpk": self.season.pk},
                    ),
                    data={
                        "orga": session.orga.pk,
                        "day": session.day,
                        "begin": "08:27",  # Modified !
                        "place": "",
                        "address_street": "",
                        "address_no": "",
                        "address_zip": "",
                        "address_city": "",
                        "address_canton": "",
                        "fallback_plan": "",
                        "superleader": "",
                        "bikes_concept": "",
                        "bikes_phone": "",
                        "apples": "",
                        "helpers_time": "07:30",
                        "helpers_place": "",
                        "comments": "",
                    },
                    follow=True,
                )

                session.refresh_from_db()
                self.assertEqual(session.begin, datetime.time(8, 27))

    def test_no_access_to_foreign_season(self):
        for symbolicurl in restrictedspecificurls:
            url = reverse(symbolicurl, kwargs={"pk": self.foreignseason.pk})
            # Final URL is NOK
            response = self.client.get(url, follow=True)
            # For helperlist and actorlist
            if symbolicurl in [
                "season-availabilities",
                "season-staff-update",
                "season-helperlist",
                "season-actorlist",
                "season-set-running",
                "season-set-open",
                "season-detail",
            ]:
                self.assertEqual(response.status_code, 403, url)
            else:
                self.assertEqual(response.status_code, 404, url)

    def test_no_access_to_foreignsession(self):
        for session in self.foreignsessions:
            urls = [
                reverse(
                    "session-list",
                    kwargs={
                        "seasonpk": self.foreignseason.pk,
                        "year": session.day.year,
                        "week": session.day.strftime("%W"),
                    },
                ),
                reverse(
                    "session-create",
                    kwargs={
                        "seasonpk": self.foreignseason.pk,
                    },
                ),
                reverse(
                    "session-detail",
                    kwargs={
                        "seasonpk": self.foreignseason.pk,
                        "pk": session.pk,
                    },
                ),
                reverse(
                    "session-update",
                    kwargs={
                        "seasonpk": self.foreignseason.pk,
                        "pk": session.pk,
                    },
                ),
                reverse(
                    "session-staff-choices",
                    kwargs={
                        "seasonpk": self.foreignseason.pk,
                        "pk": session.pk,
                    },
                ),
                reverse(
                    "session-delete",
                    kwargs={
                        "seasonpk": self.foreignseason.pk,
                        "pk": session.pk,
                    },
                ),
            ]
            for url in urls:
                # Final URL is forbidden
                response = self.client.get(url, follow=True)
                self.assertEqual(response.status_code, 403, url)

    def test_create_qualif(self):
        """
        Test `Créer une Qualif’` button
        """
        session = self.sessions[0]

        url = reverse(
            "quali-create", kwargs={"seasonpk": self.season.pk, "sessionpk": session.pk}
        )

        payload = {
            "session": session.pk,
            "name": "Classe F",
            "class_teacher_natel": "",
        }
        response = self.client.post(url, payload)
        self.assertEqual(response.status_code, 302)

    def test_m1_m2_updated(self):
        """
        Check that the m1/m2 counts are correctly updated when n_helpers is changed
        """
        url = reverse(
            "season-availabilities",
            kwargs={
                "pk": self.season.pk,
            },
        )

        # There is 4 qualifs:
        # - when n_helpers=1, m1=0 and m2=1 => We expect m1=0/0 and m2=0/4
        # - when n_helpers=3, m1=2 and m2=1 => We expect m1=0/8 and m2=0/4
        nb_qualifs = len(self.qualifs)
        for n_helpers, expected_m1, expected_m2 in [
            (1, f"0/{nb_qualifs * 0}", f"0/{nb_qualifs * 1}"),
            (3, f"0/{nb_qualifs * 2}", f"0/{nb_qualifs * 1}"),
        ]:
            for q in self.qualifs:
                q.n_helpers = n_helpers
                q.save()
            response = self.client.get(url)

            parser = BeautifulSoup(response.content, "html.parser")
            m1count = parser.select_one('[data-test-m1="%d"]' % self.sessions[0].pk)
            self.assertIsNotNone(m1count)
            self.assertEqual(str(m1count.text.strip()), expected_m1)

            m2count = parser.select_one('[data-test-m2="%d"]' % self.sessions[0].pk)
            self.assertIsNotNone(m2count)
            self.assertEqual(str(m2count.text.strip()), expected_m2)

    def test_access_to_mysession(self):
        # The season is anything but archived
        for state in DV_SEASON_STATES:
            if state[0] != DV_SEASON_STATE_ARCHIVED:
                self.season.state = state[0]
                self.season.save()
                for session in self.sessions:
                    urls = [
                        reverse(
                            "session-list",
                            kwargs={
                                "seasonpk": self.season.pk,
                                "year": session.day.year,
                                "week": session.day.strftime("%W"),
                            },
                        ),
                        reverse(
                            "session-create",
                            kwargs={
                                "seasonpk": self.season.pk,
                            },
                        ),
                        reverse(
                            "session-detail",
                            kwargs={
                                "seasonpk": self.season.pk,
                                "pk": session.pk,
                            },
                        ),
                        reverse(
                            "session-update",
                            kwargs={
                                "seasonpk": self.season.pk,
                                "pk": session.pk,
                            },
                        ),
                        reverse(
                            "session-staff-choices",
                            kwargs={
                                "seasonpk": self.season.pk,
                                "pk": session.pk,
                            },
                        ),
                        reverse(
                            "session-delete",
                            kwargs={
                                "seasonpk": self.season.pk,
                                "pk": session.pk,
                            },
                        ),
                    ]
                    for url in urls:
                        # Final URL is granted
                        response = self.client.get(url, follow=True)
                        self.assertEqual(response.status_code, 200, url)

    def test_access_to_my_archived_session(self):
        # The season is archived
        self.season.state = DV_SEASON_STATE_ARCHIVED
        self.season.save()
        for session in self.sessions:
            urls = [
                reverse(
                    "session-list",
                    kwargs={
                        "seasonpk": self.season.pk,
                        "year": session.day.year,
                        "week": session.day.strftime("%W"),
                    },
                ),
                reverse(
                    "session-detail",
                    kwargs={
                        "seasonpk": self.season.pk,
                        "pk": session.pk,
                    },
                ),
                reverse(
                    "session-staff-choices",
                    kwargs={
                        "seasonpk": self.season.pk,
                        "pk": session.pk,
                    },
                ),
            ]
            for url in urls:
                # Final URL is granted
                response = self.client.get(url, follow=True)
                self.assertEqual(response.status_code, 200, url)
        # Test the forbidden ones
        for session in self.sessions:
            urls = [
                reverse(
                    "session-create",
                    kwargs={
                        "seasonpk": self.season.pk,
                    },
                ),
                reverse(
                    "session-update",
                    kwargs={
                        "seasonpk": self.season.pk,
                        "pk": session.pk,
                    },
                ),
                reverse(
                    "session-delete",
                    kwargs={
                        "seasonpk": self.season.pk,
                        "pk": session.pk,
                    },
                ),
            ]
            for url in urls:
                # Final URL is forbidden
                response = self.client.get(url, follow=True)
                self.assertEqual(response.status_code, 403, url)

    def test_access_to_quali_views(self):
        session = self.sessions[0]
        self.assertIsNotNone(session.season.pk)
        self.assertIsNotNone(session.pk)
        # Test the Qualification creation for a session
        url = reverse(
            "quali-create",
            kwargs={
                "seasonpk": session.season.pk,
                "sessionpk": session.pk,
            },
        )
        initial = {
            "session": session.pk,
            "name": "Classe A",
            "class_teacher_natel": "",
            "n_helpers": 3,
        }
        response = self.client.post(
            url,
            initial,
        )
        self.assertEqual(response.status_code, 302, url)

        qualification = session.qualifications.first()

        # Now test the Qualification update access for a session
        url = reverse(
            "quali-update",
            kwargs={
                "seasonpk": session.season.pk,
                "sessionpk": session.pk,
                "pk": qualification.pk,
            },
        )
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200, url)

        # Test Quali update now
        initial = {
            "session": qualification.session.pk,
            "name": "Classe D",
            "n_helpers": 3,
        }
        response = self.client.post(url, initial)
        self.assertEqual(response.status_code, 302, url)


class PowerUserTest(SeasonTestCaseMixin):
    def setUp(self):
        self.client = PowerUserAuthClient()
        super(PowerUserTest, self).setUp()

    def test_access_to_season_list(self):
        for symbolicurl in restrictedgenericurls:
            url = reverse(symbolicurl)
            # Final URL is forbidden
            response = self.client.get(url, follow=True)
            self.assertEqual(response.status_code, 200, url)

    def test_access_to_season_detail(self):
        for symbolicurl in restrictedspecificurls:
            url = reverse(symbolicurl, kwargs={"pk": self.season.pk})
            # Final URL is OK
            response = self.client.get(url, follow=True)
            if "season-set-open" == symbolicurl and not self.season.can_set_state_open:
                self.assertEqual(response.status_code, 403, url)
            else:
                self.assertEqual(response.status_code, 200, url)
        for symbolicurl in restrictedhelperspecificurls:
            for helper in self.users:
                url = reverse(
                    symbolicurl, kwargs={"pk": self.season.pk, "helperpk": helper.pk}
                )
                # Final URL is OK
                response = self.client.get(url, follow=True)
                self.assertEqual(response.status_code, 200, url)

    def test_access_to_foreignsession(self):
        for session in self.foreignsessions:
            urls = [
                reverse(
                    "session-list",
                    kwargs={
                        "seasonpk": self.foreignseason.pk,
                        "year": session.day.year,
                        "week": session.day.strftime("%W"),
                    },
                ),
                reverse(
                    "session-create",
                    kwargs={
                        "seasonpk": self.foreignseason.pk,
                    },
                ),
                reverse(
                    "session-detail",
                    kwargs={
                        "seasonpk": self.foreignseason.pk,
                        "pk": session.pk,
                    },
                ),
                reverse(
                    "session-update",
                    kwargs={
                        "seasonpk": self.foreignseason.pk,
                        "pk": session.pk,
                    },
                ),
                reverse(
                    "session-staff-choices",
                    kwargs={
                        "seasonpk": self.foreignseason.pk,
                        "pk": session.pk,
                    },
                ),
                reverse(
                    "session-delete",
                    kwargs={
                        "seasonpk": self.foreignseason.pk,
                        "pk": session.pk,
                    },
                ),
            ]
            for url in urls:
                # Final URL is forbidden
                response = self.client.get(url, follow=True)
                self.assertEqual(response.status_code, 200, url)

    def test_access_to_mysession(self):
        for session in self.sessions:
            urls = [
                reverse(
                    "session-list",
                    kwargs={
                        "seasonpk": self.season.pk,
                        "year": session.day.year,
                        "week": session.day.strftime("%W"),
                    },
                ),
                reverse(
                    "session-create",
                    kwargs={
                        "seasonpk": self.season.pk,
                    },
                ),
                reverse(
                    "session-detail",
                    kwargs={
                        "seasonpk": self.season.pk,
                        "pk": session.pk,
                    },
                ),
                reverse(
                    "session-update",
                    kwargs={
                        "seasonpk": self.season.pk,
                        "pk": session.pk,
                    },
                ),
                reverse(
                    "session-staff-choices",
                    kwargs={
                        "seasonpk": self.season.pk,
                        "pk": session.pk,
                    },
                ),
                reverse(
                    "session-delete",
                    kwargs={
                        "seasonpk": self.season.pk,
                        "pk": session.pk,
                    },
                ),
            ]
            for url in urls:
                # Final URL is forbidden
                response = self.client.get(url, follow=True)
                self.assertEqual(response.status_code, 200, url)


def test_session_to_season_all_months_in_year(db):
    # A year full of seasons
    [
        SeasonFactory(cantons=["VD"], year=2019, month_start=month_start, n_months=1)
        for month_start in range(1, 13)
    ]

    session = SessionFactory(
        day=datetime.date(2019, 4, 12),
        orga=OrganizationFactory(address_canton="VD"),
    )

    assert session.season.month_start == 4


def test_session_to_season_any_month_in_year(db):
    # A year full of 3 months seasons
    SeasonFactory(cantons=["VD"], year=2019, month_start=1, n_months=3)
    SeasonFactory(cantons=["VD"], year=2019, month_start=4, n_months=3)
    SeasonFactory(cantons=["VD"], year=2019, month_start=7, n_months=3)
    SeasonFactory(cantons=["VD"], year=2019, month_start=10, n_months=3)

    for month in range(1, 13):
        session = SessionFactory(
            day=datetime.date(2019, month, 12),
            orga=OrganizationFactory(address_canton="VD"),
        )

        assert session.season.begin <= session.day
        assert session.season.end >= session.day


def test_session_to_season_in_previous_year(db):
    # Seasons crossing the year boundary
    SeasonFactory(cantons=["VD"], year=2019, month_start=7, n_months=3)
    SeasonFactory(cantons=["VD"], year=2019, month_start=10, n_months=6)
    SeasonFactory(cantons=["VD"], year=2020, month_start=4, n_months=3)

    session = SessionFactory(
        day=datetime.date(2020, 2, 12),
        orga=OrganizationFactory(address_canton="VD"),
    )

    assert session.season.begin <= session.day
    assert session.season.end >= session.day


class CoordinatorUserTest(SeasonTestCaseMixin):
    def setUp(self):
        self.client = CoordinatorAuthClient()
        super().setUp()
        # Put the sessions for our Coordinator's orga
        for s in self.sessions:
            s.orga.coordinator = self.client.user
            s.orga.save()

    def test_cooordinator_can_navigate_sessions(self):
        # Test the access to the session on all season states
        for state in DV_SEASON_STATES:
            self.season.state = state[0]
            self.season.save()
            for session in self.sessions:
                urls = [
                    reverse(
                        "session-list",
                        kwargs={
                            "seasonpk": self.season.pk,
                            "year": session.day.year,
                            "week": session.day.strftime("%W"),
                        },
                    ),
                    reverse(
                        "session-detail",
                        kwargs={
                            "seasonpk": self.season.pk,
                            "pk": session.pk,
                        },
                    ),
                ]
                for url in urls:
                    # Coordinators can navigate the seasons in all states
                    response = self.client.get(url)
                    self.assertEqual(response.status_code, 200, url)

    def test_coordinator_can_update_sessions(self):
        # Test the access to the session on all season states
        for state in DV_SEASON_STATES:
            self.season.state = state[0]
            self.season.save()
            for session in self.sessions:
                url = reverse(
                    "session-update",
                    kwargs={
                        "seasonpk": self.season.pk,
                        "pk": session.pk,
                    },
                )
                response = self.client.get(
                    url,
                    follow=True,
                )
                if state[0] in [DV_SEASON_STATE_PLANNING, DV_SEASON_STATE_OPEN]:
                    # Coordinator can update sessions only in planning
                    self.assertEqual(response.status_code, 200, url)
                else:
                    self.assertEqual(response.status_code, 403, url)

    def test_coordinator_cannot_create_nor_delete_sessions(self):
        # Test the access to the session on all season states
        for state in DV_SEASON_STATES:
            self.season.state = state[0]
            self.season.save()
            for session in self.sessions:
                urls = [
                    reverse(
                        "session-create",
                        kwargs={
                            "seasonpk": self.season.pk,
                        },
                    ),
                    reverse(
                        "session-delete",
                        kwargs={
                            "seasonpk": self.season.pk,
                            "pk": session.pk,
                        },
                    ),
                ]
                for url in urls:
                    # Final URL is forbidden; no creation nor deletion by coordinators
                    response = self.client.get(url, follow=True)
                    self.assertEqual(response.status_code, 403, url)


class TestPlanning(SeasonTestCaseMixin):
    canton = "VD"

    def setUp(self):
        self.client = StateManagerAuthClient()

        # Make sure the current user is in VD
        self.client.user.profile.affiliation_canton = self.canton
        self.client.user.profile.save()

        # User that we want to check the individual planning for
        self.user1 = UserFactory(
            profile__affiliation_canton=self.canton, profile__formation=FORMATION_M2
        )

        super().setUp()

    def assertCell(
        self, response, user, session, title, class_selector=".glyphicon-remove-sign"
    ):
        key = AVAILABILITY_FIELDKEY.format(hpk=user.pk, spk=session.pk)

        parser = BeautifulSoup(response.content, "html.parser")
        cell = parser.select_one(f'[data-test="{key}"] {class_selector}')
        self.assertIsNotNone(cell, "Cell not found in the planning")
        self.assertEqual(cell.attrs.get("title"), title)

    def test_individual_planning_view_not_chosen(self):
        """
        Test that when the user's availability is okayish for a session, but they are not selected, the planning cell is red.
        """

        for s in self.sessions:
            # Put the sessions for our Coordinator's orga and canton
            s.canton = self.canton
            s.orga.address_canton = self.canton
            s.orga.coordinator = self.client.user
            s.orga.save()
            s.save()

        # Set the user availability for that session to "if needed"
        HelperSessionAvailability.objects.update_or_create(
            session=self.sessions[0], helper=self.user1, defaults={"availability": "i"}
        )

        # Choose someone else
        HelperSessionAvailability.objects.create(
            session=self.sessions[0],
            helper=self.users[1],
            availability=CHOSEN_AS_LEADER,
        )

        url = reverse(
            "season-planning", kwargs={"helperpk": self.user1.pk, "pk": self.season.pk}
        )
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)
        self.assertCell(response, self.user1, self.sessions[0], _("Pas choisi"))

    def test_individual_planning_view_no(self):
        """
        Test that when the user's availability is "no" for a session the planning cell is red.
        """
        for s in self.sessions:
            # Put the sessions for our Coordinator's orga and canton
            s.canton = self.canton
            s.orga.address_canton = self.canton
            s.orga.coordinator = self.client.user
            s.orga.save()
            s.save()
        # Set the user availability for that session to No
        HelperSessionAvailability.objects.update_or_create(
            session=self.sessions[0], helper=self.user1, defaults={"availability": "n"}
        )

        url = reverse(
            "season-planning", kwargs={"helperpk": self.user1.pk, "pk": self.season.pk}
        )
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)
        self.assertCell(response, self.user1, self.sessions[0], _("Non"))

    def test_general_planning_view(self):
        # Build URL for aggregated general planning
        year = self.season.year
        dv_season = (
            DV_SEASON_SPRING
            if self.season.month_start <= DV_SEASON_LAST_SPRING_MONTH
            else DV_SEASON_AUTUMN
        )
        # Pick a session in current canton/year and ensure helper is assigned and has an availability
        session_in_scope = self.sessions[0]
        session_in_scope.orga.address_canton = self.canton
        session_in_scope.orga.save()
        QualificationFactory(actor=self.user1, session=session_in_scope)
        HelperSessionAvailability.objects.update_or_create(
            session=session_in_scope, helper=self.user1, defaults={"availability": "i"}
        )

        url = reverse(
            "season-general-planning",
            kwargs={
                "year": year,
                "dv_season": dv_season,
                "helperpk": self.user1.pk,
            },
        )

        response = self.client.get(url)
        self.assertEqual(response.status_code, 200, url)

        parser = BeautifulSoup(response.content, "html.parser")
        header_cell = parser.select_one(f'th[data-test="orga-{session_in_scope.pk}"]')
        self.assertIsNotNone(
            header_cell,
            f"Expected header cell for orga-{session_in_scope.pk} to be present",
        )

    def test_general_planning_redirect_helperpk_zero(self):
        # helperpk=0 should redirect to the same view with the current user's pk
        year = self.season.year
        dv_season = (
            DV_SEASON_SPRING
            if self.season.month_start <= DV_SEASON_LAST_SPRING_MONTH
            else DV_SEASON_AUTUMN
        )

        url = reverse(
            "season-general-planning",
            kwargs={
                "year": year,
                "dv_season": dv_season,
                "helperpk": 0,
            },
        )

        response = self.client.get(url)
        self.assertEqual(response.status_code, 302, url)

    def test_general_personal_planning_export(self):
        # Export aggregated personal planning for the chosen helper
        year = self.season.year
        dv_season = (
            DV_SEASON_SPRING
            if self.season.month_start <= DV_SEASON_LAST_SPRING_MONTH
            else DV_SEASON_AUTUMN
        )
        # Ensure helper is assigned to an in-scope session with correct canton
        session_in_scope = self.sessions[0]
        session_in_scope.orga.address_canton = self.canton
        session_in_scope.orga.save()
        QualificationFactory(actor=self.user1, session=session_in_scope)

        for exportformat in ["csv", "ods", "xls"]:
            url = reverse(
                "season-personal-planning-export",
                kwargs={
                    "year": year,
                    "dv_season": dv_season,
                    "helperpk": self.user1.pk,
                    "format": exportformat,
                },
            )
            response = self.client.get(url)
            self.assertEqual(response.status_code, 200, url)
            if exportformat == "csv":
                content = response.content.decode("utf-8")
                self.assertIn(
                    session_in_scope.orga.name,
                    content,
                    "Expected organiser name to be present in CSV export",
                )



# Generated by Django 2.2.24 on 2022-01-19 16:50

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("salary", "0015_add_zoug"),
    ]

    operations = [
        migrations.AddField(
            model_name="monthlycantonalvalidationurltranslation",
            name="label",
            field=models.CharField(blank=True, max_length=255, verbose_name="Label"),
        ),
    ]



# Generated by Django 1.11.4 on 2017-08-22 13:10
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("user", "0036_userprofile_tax_jurisdiction"),
    ]

    operations = [
        migrations.AlterModelManagers(
            name="userprofile",
            managers=[],
        ),
    ]



# Generated by Django 4.2.6 on 2025-09-24 14:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('challenge', '0084_season_availability_reminder_sent_at'),
    ]

    operations = [
        migrations.AddField(
            model_name='historicalseason',
            name='availability_reminder_sent_at',
            field=models.DateTimeField(blank=True, null=True, verbose_name='Date d’envoi du rappel de disponibilités'),
        ),
    ]



# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2015 Didier Raboud <me+defivelo@odyx.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re

from django.test import TestCase
from django.urls import reverse

from apps.common import DV_STATES
from defivelo.tests.utils import (
    AuthClient,
    CoordinatorAuthClient,
    PowerUserAuthClient,
    StateManagerAuthClient,
    SuperUserAuthClient,
)

from ..models import ORGASTATUS_ACTIVE, Organization
from .factories import OrganizationFactory


class OrgaBasicTest(TestCase):
    def setUp(self):
        # Every test needs a client.
        self.client = AuthClient()
        self.expected_code = 403
        self.expected_save_code = 403
        self.expect_templates = False

        self.orga = OrganizationFactory()
        self.orga.save()

    def test_access_to_orga_list(self):
        response = self.client.get(reverse("organization-list"))
        if self.expect_templates:
            self.assertTemplateUsed(response, "orga/organization_filter.html")
        self.assertEqual(response.status_code, self.expected_code)

        response = self.client.get(
            reverse("organization-list-export", kwargs={"format": "csv"})
        )
        self.assertEqual(response.status_code, self.expected_code)

    def test_access_to_orga_detail(self):
        # Issue a GET request.
        response = self.client.get(
            reverse("organization-detail", kwargs={"pk": self.orga.pk})
        )

        if self.expect_templates:
            self.assertTemplateUsed(response, "orga/organization_detail.html")
        self.assertEqual(response.status_code, self.expected_code)

    def test_access_to_orga_create(self):
        url = reverse("organization-create")
        response = self.client.get(url)

        if self.expect_templates:
            self.assertTemplateUsed(response, "orga/organization_form.html")
        self.assertEqual(response.status_code, self.expected_code)

    def test_access_to_orga_edit(self):
        url = reverse("organization-update", kwargs={"pk": self.orga.pk})
        response = self.client.get(url)

        if self.expect_templates:
            self.assertTemplateUsed(response, "orga/organization_form.html")
        self.assertEqual(response.status_code, self.expected_code)

    def test_autocompletes(self):
        url = reverse("organization-autocomplete")
        response = self.client.get(url)
        self.assertEqual(response.status_code, self.expected_code, url)


class OrgaPowerUserTest(OrgaBasicTest):
    def setUp(self):
        super(OrgaPowerUserTest, self).setUp()
        self.client = PowerUserAuthClient()
        self.expected_code = 200
        self.expected_save_code = 302
        self.expect_templates = True

    def test_access_to_orga_create(self):
        super().test_access_to_orga_create()

        url = reverse("organization-create")
        initial = {
            "name": "Orga",
            "status": ORGASTATUS_ACTIVE,
        }

        # Test without a canton, we expect an error, that's a return to the page (200)
        response = self.client.post(url, initial)
        self.assertEqual(response.status_code, 200, url)
        self.assertEqual(Organization.objects.count(), 1)  # There is one in the setUp

        # Test with a canton, that must work.
        initial["address_canton"] = "VD"
        response = self.client.post(url, initial)
        self.assertEqual(response.status_code, self.expected_save_code, url)
        self.assertEqual(Organization.objects.count(), 2)  # There is one in the setUp
        self.assertEqual(Organization.objects.order_by("-id")[0].address_canton, "VD")

    def test_access_to_orga_edit(self):
        url = reverse("organization-update", kwargs={"pk": self.orga.pk})
        super(OrgaPowerUserTest, self).test_access_to_orga_edit()

        self.orga.address_canton = "VD"
        self.orga.save()

        initial = self.orga.__dict__.copy()
        del initial["id"]
        del initial["created_on"]
        del initial["address_ptr_id"]
        del initial["_state"]

        # Test some update, that must go through
        initial["address_canton"] = "JU"

        for key in initial:
            initial[key] = "" if initial[key] is None else initial[key]

        response = self.client.post(url, initial)
        self.assertEqual(response.status_code, self.expected_save_code, url)

        neworga = Organization.objects.get(pk=self.orga.id)
        self.assertEqual(neworga.address_canton, "JU")


class SuperUserTest(OrgaBasicTest):
    def setUp(self):
        super(SuperUserTest, self).setUp()
        self.client = SuperUserAuthClient()
        self.expected_code = 200
        self.expected_save_code = 403
        self.expect_templates = True
        self.orgas = [OrganizationFactory(address_canton=c) for c in DV_STATES]

    def test_autocompletes(self):
        url = reverse("organization-autocomplete")
        response = self.client.get(url)
        self.assertEqual(response.status_code, self.expected_code, url)
        # Check that we only find our orga
        entries = [int(d) for d in re.findall(r'"id": "(\d+)"', str(response.content))]
        entries.sort()
        allentries = [self.orga.pk] + [o.pk for o in self.orgas]
        if re.search('"pagination": {"more": true}', str(response.content)):
            self.assertTrue(set(entries).issubset(allentries))
        else:
            self.assertEqual(entries, allentries)

    def test_accented_search(self):
        self.orga.abbr = "ÉCCG"
        self.orga.save()
        response = self.client.get("%s?%s" % (reverse("organization-list"), "q=eccg"))
        self.assertContains(response, "ÉCCG")


class OrgaStateManagerUserTest(TestCase):
    expected_code = 200

    def setUp(self):
        self.client = StateManagerAuthClient()
        mycanton = str((self.client.user.managedstates.first()).canton)
        self.myorga = OrganizationFactory(address_canton=mycanton)
        self.myorga.save()

        self.OTHERSTATES = [c for c in DV_STATES if c != mycanton]
        self.foreignorga = OrganizationFactory(address_canton=self.OTHERSTATES[0])
        self.foreignorga.save()

    def test_access_to_orga_list(self):
        response = self.client.get(reverse("organization-list"))
        self.assertTemplateUsed(response, "orga/organization_filter.html")
        self.assertEqual(response.status_code, self.expected_code)

        response = self.client.get(
            reverse("organization-list-export", kwargs={"format": "csv"})
        )
        self.assertEqual(response.status_code, self.expected_code)

    def test_access_to_orga_detail(self):
        response = self.client.get(
            reverse("organization-detail", kwargs={"pk": self.myorga.pk})
        )
        self.assertTemplateUsed(response, "orga/organization_detail.html")
        self.assertEqual(response.status_code, self.expected_code)

        # The other orga can also be accessed
        response = self.client.get(
            reverse("organization-detail", kwargs={"pk": self.foreignorga.pk})
        )
        self.assertEqual(response.status_code, 200)

    def test_access_to_orga_create(self):
        url = reverse("organization-create")
        initial = {
            "name": "Orga",
            "status": ORGASTATUS_ACTIVE,
            "address_canton": self.client.user.managedstates.first().canton,
        }

        # One of our cantons is mandatory
        response = self.client.post(url, initial)
        self.assertEqual(response.status_code, 302, url)
        self.assertEqual(Organization.objects.count(), 3)  # There are two in the setUp
        self.assertEqual(
            Organization.objects.order_by("-id")[0].address_canton,
            self.client.user.managedstates.first().canton,
        )

        initial = {
            "name": "Orga",
            "status": ORGASTATUS_ACTIVE,
            "address_canton": self.OTHERSTATES[0],
        }

        # A creation outside our cantons is not OK
        response = self.client.post(url, initial)
        # Code 200 because create failed
        self.assertEqual(response.status_code, 200, url)
        self.assertEqual(
            Organization.objects.count(), 3
        )  # There are two in the setUp, and one above

    def test_access_to_orga_edit(self):
        url = reverse("organization-update", kwargs={"pk": self.myorga.pk})
        response = self.client.get(url)
        self.assertTemplateUsed(response, "orga/organization_form.html")
        self.assertEqual(response.status_code, self.expected_code)

        # Our orga cannot be edited away from my cantons
        initial = self.myorga.__dict__.copy()
        del initial["id"]
        del initial["created_on"]
        del initial["address_ptr_id"]
        del initial["_state"]

        initial["address_no"] = self.myorga.address_no + 1

        for key in initial:
            initial[key] = "" if initial[key] is None else initial[key]

        response = self.client.post(url, initial)
        # Code 302 because update succeeded
        self.assertEqual(response.status_code, 302, url)
        # Check update succeeded
        neworga = Organization.objects.get(pk=self.myorga.pk)
        self.assertEqual(neworga.address_no, str(self.myorga.address_no + 1))

        # Test some update, that must go through
        initial["address_canton"] = self.foreignorga.address_canton

        response = self.client.post(url, initial)
        # Code 200 because update failed
        self.assertEqual(response.status_code, 200, url)
        # Check update failed
        neworga = Organization.objects.get(pk=self.myorga.pk)
        self.assertEqual(neworga.address_canton, self.myorga.address_canton)

        # The other orga cannot be accessed
        response = self.client.get(
            reverse("organization-update", kwargs={"pk": self.foreignorga.pk})
        )
        self.assertEqual(response.status_code, 404)

    def test_autocompletes(self):
        url = reverse("organization-autocomplete")
        response = self.client.get(url)
        self.assertEqual(response.status_code, self.expected_code, url)
        # Check that we find all orgas, including orgas of other cantons
        entries = re.findall(r'"id": "(\d+)"', str(response.content))
        expected_entries = set(str(o.id) for o in Organization.objects.all())
        self.assertEqual(set(entries), expected_entries)


class OrgaCoordinatorUserTest(TestCase):
    expected_code = 200

    def setUp(self):
        self.client = CoordinatorAuthClient()
        self.myorga = OrganizationFactory(coordinator=self.client.user)
        self.myorga.save()

        self.foreignorga = OrganizationFactory()
        self.foreignorga.save()

    def test_access_to_orga_list(self):
        response = self.client.get(reverse("organization-list"))
        self.assertTemplateUsed(response, "orga/organization_filter.html")
        self.assertEqual(response.status_code, self.expected_code)

        response = self.client.get(
            reverse("organization-list-export", kwargs={"format": "csv"})
        )
        self.assertEqual(response.status_code, self.expected_code)

    def test_access_to_orga_detail(self):
        response = self.client.get(
            reverse("organization-detail", kwargs={"pk": self.myorga.pk})
        )
        self.assertTemplateUsed(response, "orga/organization_detail.html")
        self.assertEqual(response.status_code, self.expected_code)

        # The other orga can also be accessed
        response = self.client.get(
            reverse("organization-detail", kwargs={"pk": self.foreignorga.pk})
        )
        self.assertEqual(response.status_code, 200)

    def test_access_to_orga_edit(self):
        url = reverse("organization-update", kwargs={"pk": self.myorga.pk})
        response = self.client.get(url)
        self.assertTemplateUsed(response, "orga/organization_form.html")
        self.assertEqual(response.status_code, self.expected_code)

        # Our orga cannot be edited away from my cantons
        initial = self.myorga.__dict__.copy()
        del initial["id"]
        del initial["created_on"]
        del initial["address_ptr_id"]
        del initial["_state"]

        initial["address_no"] = self.myorga.address_no + 1

        for key in initial:
            initial[key] = "" if initial[key] is None else initial[key]

        response = self.client.post(url, initial)
        # Code 302 because update succeeded
        self.assertEqual(response.status_code, 302, url)
        # Check update succeeded
        neworga = Organization.objects.get(pk=self.myorga.pk)
        self.assertEqual(neworga.address_no, str(self.myorga.address_no + 1))

        # Test some update, that must go through
        initial["coordinator"] = self.client.user

        response = self.client.post(url, initial)
        # Check update failed
        neworga = Organization.objects.get(pk=self.myorga.pk)
        self.assertEqual(neworga.coordinator, self.myorga.coordinator)

        # The other orga cannot be accessed
        response = self.client.get(
            reverse("organization-update", kwargs={"pk": self.foreignorga.pk})
        )
        self.assertEqual(response.status_code, 404)



# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2015 Didier Raboud <me+defivelo@odyx.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import re

from django.contrib.auth import get_user_model
from django.core import mail
from django.test import TestCase, override_settings
from django.urls import NoReverseMatch, reverse

from rolepermissions.roles import get_user_roles

from apps.common import DV_STATES
from apps.user import FORMATION_M1, FORMATION_M2
from apps.user.models import (
    BAGSTATUS_LOAN,
    BAGSTATUS_NONE,
    BAGSTATUS_PAID,
    STD_PROFILE_FIELDS,
    USERSTATUS_ACTIVE,
    USERSTATUS_INACTIVE,
)
from apps.user.tests.factories import UserFactory
from defivelo.tests.utils import (
    AuthClient,
    CollaboratorAuthClient,
    CoordinatorAuthClient,
    PowerUserAuthClient,
    StateManagerAuthClient,
    SuperUserAuthClient,
)

User = get_user_model()

myurlsforall = [
    "user-detail",
    "user-update",
    "profile-detail",
]
myurls_for_office_and_collaborators = [
    "user-list",
    "user-list-export",
]

othersurls = [
    "user-detail",
    "user-update",
    "user-create",
    "user-sendcredentials",
    "user-delete",
]

profile_autocompletes = [
    "Actors",
    "AllPersons",
    "Leaders",
    "Helpers",
    "PersonsRelevantForSessions",
]


def tryurl(symbolicurl, user, exportformat="csv"):
    try:
        try:
            url = reverse(symbolicurl, kwargs={"pk": user.pk})
        except NoReverseMatch:
            url = reverse(symbolicurl, kwargs={"format": exportformat})
    except NoReverseMatch:
        url = reverse(symbolicurl)
    return url


class ProfileTestCase(TestCase):
    def setUp(self):
        # Every test needs a client.
        self.client = AuthClient()
        self.users = [UserFactory(profile__affiliation_canton="NE") for i in range(3)]

    def getprofileinitial(self, user):
        userfields = ["first_name", "last_name", "email"]

        initial = {k: v for (k, v) in user.__dict__.items() if k in userfields}
        initial.update(
            {
                k: v
                for (k, v) in user.profile.__dict__.items()
                if k in STD_PROFILE_FIELDS
            }
        )

        # Some corrections
        initial["language"] = "fr"
        initial["status"] = 0
        initial["birthdate"] = ""
        initial["formation_firstdate"] = ""
        initial["formation_lastdate"] = ""

        if not initial["activity_cantons"]:
            initial["activity_cantons"] = []

        if not initial["languages_challenges"]:
            initial["languages_challenges"] = []

        return initial


class AuthUserTest(ProfileTestCase):
    def test_my_allowances(self):
        for symbolicurl in myurlsforall:
            url = tryurl(symbolicurl, self.client.user)
            response = self.client.get(url)
            self.assertEqual(response.status_code, 200, url)

    def test_my_restrictions(self):
        for symbolicurl in myurls_for_office_and_collaborators + [
            "user-assign-role",
        ]:
            for exportformat in ["csv", "ods", "xls"]:
                url = tryurl(symbolicurl, self.client.user, exportformat)
                response = self.client.get(url)
                self.assertEqual(response.status_code, 403, url)

    def test_otherusers_access(self):
        for symbolicurl in othersurls + [
            "user-assign-role",
        ]:
            for otheruser in self.users:
                url = tryurl(symbolicurl, otheruser)
                response = self.client.get(url)
                self.assertEqual(response.status_code, 403, url)

    def test_my_profile_access(self):
        # Pre-update profile and user
        self.client.user.profile.formation = FORMATION_M1
        self.client.user.profile.affiliation_canton = "GE"
        self.client.user.profile.bagstatus = BAGSTATUS_LOAN
        self.client.user.profile.status = USERSTATUS_ACTIVE
        self.client.user.profile.language = "fr"
        self.client.user.profile.employee_code = "poor"
        self.client.user.profile.save()
        url = reverse("user-update", kwargs={"pk": self.client.user.pk})
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200, url)

        initial = self.getprofileinitial(self.client.user)
        # Test some update, that must go through
        initial["first_name"] = "newfirstname"
        initial["activity_cantons"] = [
            "JU",
            "GE",
            "VD",
        ]
        initial["language"] = "de"
        initial["languages_challenges"] = [
            "de",
            "fr",
        ]
        initial["status"] = USERSTATUS_INACTIVE
        initial["bank_name"] = "Banque Alternative Suisse, succursale de Lausanne"

        # And some that mustn't
        initial["formation"] = FORMATION_M2
        initial["affiliation_canton"] = "VD"
        initial["bagstatus"] = BAGSTATUS_PAID
        initial["employee_code"] = "I want to be rich"

        response = self.client.post(url, initial)
        self.assertEqual(response.status_code, 302, url)

        # Get our user from DB
        me = get_user_model().objects.get(pk=self.client.user.pk)

        # Updated
        self.assertEqual(me.first_name, "newfirstname")
        self.assertEqual(
            me.profile.activity_cantons,
            [
                "JU",
                "VD",
            ],
        )
        self.assertEqual(me.profile.language, "de")
        self.assertEqual(
            me.profile.languages_challenges,
            [
                "fr",
            ],
        )
        self.assertEqual(me.profile.status, USERSTATUS_INACTIVE)
        self.assertEqual(
            me.profile.bank_name, "Banque Alternative Suisse, succursale de Lausanne"
        )

        # Not updated
        self.assertEqual(me.profile.employee_code, "poor")
        self.assertEqual(me.profile.formation, FORMATION_M1)
        self.assertEqual(me.profile.bagstatus, BAGSTATUS_LOAN)
        self.assertEqual(me.profile.affiliation_canton, "GE")

    def test_autocompletes(self):
        # All autocompletes are forbidden
        for al in profile_autocompletes:
            url = "%s?q=test" % reverse("user-%s-ac" % al)
            response = self.client.get(url)
            self.assertEqual(response.status_code, 403, url)


class CollaboratorUserTest(AuthUserTest):
    def setUp(self):
        super().setUp()
        self.client = CollaboratorAuthClient()

    def test_my_allowances(self):
        for symbolicurl in myurlsforall + myurls_for_office_and_collaborators:
            for exportformat in ["csv", "ods", "xls"]:
                url = tryurl(symbolicurl, self.client.user, exportformat)
                response = self.client.get(url)
                self.assertEqual(response.status_code, 200, url)

    def test_my_restrictions(self):
        # Collaborators can't assign roles.
        url = tryurl(
            "user-assign-role",
            self.client.user,
        )
        response = self.client.get(url)
        self.assertEqual(response.status_code, 403, url)

    def test_autocompletes(self):
        # Autocompletes are OK for collaborators, they have access to the user list.
        for al in profile_autocompletes:
            url = "%s?q=test" % reverse("user-%s-ac" % al)
            response = self.client.get(url)
            self.assertEqual(response.status_code, 200, url)


class PowerUserTest(ProfileTestCase):
    def setUp(self):
        super(PowerUserTest, self).setUp()
        self.client = PowerUserAuthClient()

    def test_my_allowances(self):
        for symbolicurl in myurlsforall + myurls_for_office_and_collaborators:
            for exportformat in ["csv", "ods", "xls"]:
                url = tryurl(symbolicurl, self.client.user, exportformat)
                response = self.client.get(url)
                self.assertEqual(response.status_code, 200, url)

    def test_otherusers_access(self):
        for symbolicurl in othersurls:
            for otheruser in self.users:
                url = tryurl(symbolicurl, otheruser)
                response = self.client.get(url)
                self.assertEqual(response.status_code, 200, url)

    @override_settings(PROFILE_CHANGED_NOTIFY_EMAIL=None)
    def test_send_creds(self):
        nmails = 0
        for otheruser in self.users:
            url = tryurl("user-sendcredentials", otheruser)
            response = self.client.get(url)
            self.assertEqual(response.status_code, 200, url)
            # Now post to it, to get the mail sent
            response = self.client.post(url, {})
            self.assertEqual(response.status_code, 302, url)

            nmails += 1
            self.assertEqual(len(mail.outbox), nmails)

            # Verify what they are from the DB
            dbuser = get_user_model().objects.get(pk=otheruser.pk)
            self.assertTrue(dbuser.is_active)
            self.assertTrue(dbuser.has_usable_password())
            self.assertTrue(dbuser.profile.can_login)
            self.assertFalse(dbuser.profile.deleted)

            # Second try should fail, now that each of the users has a
            # a valid email and got a password sent
            response = self.client.get(url)
            self.assertEqual(response.status_code, 403, url)

            # Allowed to re-send creds though, any number of times
            for i in range(2):
                url = tryurl("user-resendcredentials", otheruser)
                response = self.client.get(url)
                self.assertEqual(response.status_code, 200, url)

                response = self.client.post(url, {})
                self.assertEqual(response.status_code, 302, url)

                nmails += 1
                self.assertEqual(len(mail.outbox), nmails)

            url = tryurl("user-delete", otheruser)
            response = self.client.get(url)
            self.assertEqual(response.status_code, 200, url)
            # Now post to it, to remove the user
            response = self.client.post(url, {})
            self.assertEqual(response.status_code, 302, url)
            # Get again
            dbuser = get_user_model().objects.get(pk=otheruser.pk)

            self.assertFalse(dbuser.profile.can_login)
            self.assertTrue(dbuser.profile.deleted)

    def test_other_profile_accesses(self):
        for user in self.users:
            # Pre-update profile and user
            user.profile.formation = FORMATION_M1
            user.profile.employee_code = "rich"
            user.profile.save()
            url = reverse("user-update", kwargs={"pk": user.pk})
            response = self.client.get(url)
            self.assertEqual(response.status_code, 200, url)

            initial = self.getprofileinitial(user)
            # Test some update, that must go through
            initial["first_name"] = "newfirstname"
            initial["activity_cantons"] = [
                "JU",
                "VD",
                "GE",
            ]
            initial["employee_code"] = "poor"
            initial["formation"] = FORMATION_M2
            initial["affiliation_canton"] = "VD"

            response = self.client.post(url, initial)
            self.assertEqual(response.status_code, 302, url)

            # Get our user from DB
            her = get_user_model().objects.get(pk=user.pk)

            # Updated
            self.assertEqual(her.first_name, "newfirstname")
            # Pas de VD parce que le canton d’affiliation est 'VD'
            self.assertEqual(
                her.profile.activity_cantons,
                [
                    "JU",
                    "GE",
                ],
            )

            # Updated as well
            self.assertEqual(her.profile.formation, FORMATION_M2)
            self.assertEqual(her.profile.affiliation_canton, "VD")
            self.assertEqual(her.profile.employee_code, "poor")

    def test_roleassign(self):
        # Can't change my own role
        url = reverse("user-assign-role", kwargs={"pk": self.client.user.pk})
        response = self.client.get(url)
        self.assertEqual(response.status_code, 403, response)

        # But I can change any other role
        user = self.users[0]
        # Make this user a without-role user
        user.profile.formation = ""
        user.profile.actor_for.clear()
        user.profile.save()

        url = reverse("user-assign-role", kwargs={"pk": user.pk})
        response = self.client.get(url)
        # That user has no login
        self.assertEqual(response.status_code, 403, response)

        url = tryurl("user-sendcredentials", user)
        # Now post to it, to get the mail sent
        response = self.client.post(url, {})
        self.assertEqual(response.status_code, 302, url)

        url = reverse("user-assign-role", kwargs={"pk": user.pk})
        response = self.client.get(url)
        self.assertTemplateUsed(response, "roles/assign.html")
        self.assertEqual(response.status_code, 200, response)
        self.assertEqual(get_user_roles(user), [], user)
        response = self.client.post(
            url,
            {
                "role": "state_manager",
                "managed_states": [
                    "VD",
                ],
            },
        )
        self.assertEqual(response.status_code, 302, url)
        self.assertEqual(
            list(user.managedstates.all().values_list("canton", flat=True)),
            ["VD"],
            user,
        )
        self.assertEqual(
            [r.get_name() for r in get_user_roles(user)], ["state_manager"], user
        )

        response = self.client.post(
            url,
            {
                "role": "power_user",
                "managed_states": [
                    "VD",
                ],
            },
        )
        self.assertEqual(response.status_code, 302, url)
        # No canton for non-state_manager
        self.assertEqual(
            list(user.managedstates.all().values_list("canton", flat=True)), [], user
        )
        self.assertEqual(
            [r.get_name() for r in get_user_roles(user)], ["power_user"], user
        )

        # Deleting the role also works
        response = self.client.post(url, {"role": "", "managed_states": ["VD"]})
        self.assertEqual(response.status_code, 302, url)
        self.assertEqual(get_user_roles(user), [], user)
        # No canton for non-state_manager
        self.assertEqual(
            list(user.managedstates.all().values_list("canton", flat=True)), [], user
        )

    def test_autocompletes(self):
        # All autocompletes are permitted
        for al in profile_autocompletes:
            url = "%s?q=test" % reverse("user-%s-ac" % al)
            response = self.client.get(url)
            self.assertEqual(response.status_code, 200, url)


class CoordinatorUserTest(ProfileTestCase):
    def setUp(self):
        super().setUp()
        self.client = CoordinatorAuthClient()
        for user in self.users:
            user.profile.save()

    def test_my_allowances(self):
        for symbolicurl in myurlsforall:
            for exportformat in ["csv", "ods", "xls"]:
                url = tryurl(symbolicurl, self.client.user, exportformat)
                response = self.client.get(url)
                self.assertEqual(response.status_code, 200, url)

    def test_my_profile_access(self):
        # Pre-update profile and user
        self.client.user.profile.formation = FORMATION_M1
        self.client.user.profile.affiliation_canton = "GE"
        self.client.user.profile.bagstatus = BAGSTATUS_LOAN
        self.client.user.profile.status = USERSTATUS_ACTIVE
        self.client.user.profile.language = "fr"
        self.client.user.profile.employee_code = "poor"
        self.client.user.profile.save()
        url = reverse("user-update", kwargs={"pk": self.client.user.pk})
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200, url)

        initial = self.getprofileinitial(self.client.user)
        # Test some update, that must go through
        initial["first_name"] = "newfirstname"
        initial["activity_cantons"] = [
            "JU",
            "GE",
            "VD",
        ]
        initial["language"] = "de"
        initial["languages_challenges"] = [
            "de",
            "fr",
        ]
        initial["status"] = USERSTATUS_INACTIVE
        initial["bank_name"] = "Banque Alternative Suisse, succursale de Lausanne"

        # And some that mustn't
        initial["formation"] = FORMATION_M2
        initial["affiliation_canton"] = "VD"
        initial["bagstatus"] = BAGSTATUS_PAID
        initial["employee_code"] = "I want to be rich"

        response = self.client.post(url, initial)
        self.assertEqual(response.status_code, 302, url)

        # Get our user from DB
        me = get_user_model().objects.get(pk=self.client.user.pk)

        # Updated
        self.assertEqual(me.first_name, "newfirstname")
        self.assertEqual(
            me.profile.activity_cantons,
            [
                "JU",
                "VD",
            ],
        )
        self.assertEqual(me.profile.language, "de")
        self.assertEqual(
            me.profile.languages_challenges,
            [
                "fr",
            ],
        )
        self.assertEqual(me.profile.status, USERSTATUS_INACTIVE)
        self.assertEqual(
            me.profile.bank_name, "Banque Alternative Suisse, succursale de Lausanne"
        )

        # Not updated
        self.assertEqual(me.profile.employee_code, "poor")
        self.assertEqual(me.profile.formation, FORMATION_M1)
        self.assertEqual(me.profile.bagstatus, BAGSTATUS_LOAN)
        self.assertEqual(me.profile.affiliation_canton, "GE")

    def test_my_simple_profile_access(self):
        # Pre-update profile and user
        self.client.user.profile.status = USERSTATUS_ACTIVE
        self.client.user.profile.language = "fr"
        self.client.user.profile.save()
        url = reverse("user-update", kwargs={"pk": self.client.user.pk})
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200, url)

        initial = self.getprofileinitial(self.client.user)
        # Test some update, that must go through
        initial["first_name"] = "newfirstname"
        initial["activity_cantons"] = [
            "JU",
            "GE",
            "VD",
        ]
        initial["language"] = "de"
        initial["languages_challenges"] = [
            "de",
            "fr",
        ]
        initial["status"] = USERSTATUS_INACTIVE
        initial["bank_name"] = "Banque Alternative Suisse, succursale de Lausanne"

        # And some that mustn't
        initial["formation"] = FORMATION_M2
        initial["affiliation_canton"] = "VD"
        initial["bagstatus"] = BAGSTATUS_PAID
        initial["employee_code"] = "I want to be rich"

        response = self.client.post(url, initial)
        self.assertEqual(response.status_code, 302, url)

        # Get our user from DB
        me = get_user_model().objects.get(pk=self.client.user.pk)

        # Updated
        self.assertEqual(me.first_name, "newfirstname")
        self.assertEqual(me.profile.language, "de")

        # Not updated
        self.assertEqual(me.profile.bank_name, "")
        self.assertEqual(me.profile.languages_challenges, [])
        self.assertEqual(me.profile.activity_cantons, [])
        self.assertEqual(me.profile.employee_code, "")
        self.assertEqual(me.profile.status, USERSTATUS_ACTIVE)
        self.assertEqual(me.profile.formation, "")
        self.assertEqual(me.profile.bagstatus, BAGSTATUS_NONE)
        self.assertEqual(me.profile.affiliation_canton, "")


class StateManagerUserTest(ProfileTestCase):
    def setUp(self):
        super(StateManagerUserTest, self).setUp()
        self.client = StateManagerAuthClient()
        mycanton = str((self.client.user.managedstates.first()).canton)

        OTHERSTATES = [c for c in DV_STATES if c != mycanton]
        for user in self.users:
            user.profile.affiliation_canton = OTHERSTATES[0]
            user.profile.employee_code = "poor"
            user.profile.save()

        self.users[0].profile.affiliation_canton = mycanton
        self.users[0].profile.save()

        self.myuser = self.users[0]
        self.foreignuser = self.users[1]

    def test_accented_search(self):
        self.users[0].first_name = "Joël"
        self.users[0].save()
        response = self.client.get("%s?%s" % (reverse("user-list"), "q=joel"))
        self.assertContains(response, "Joël")

    def test_my_allowances(self):
        for symbolicurl in myurlsforall + myurls_for_office_and_collaborators:
            for exportformat in ["csv", "ods", "xls"]:
                url = tryurl(symbolicurl, self.client.user, exportformat)
                response = self.client.get(url)
                self.assertEqual(response.status_code, 200, url)

    def test_otherusers_access(self):
        response = self.client.get(
            reverse("user-detail", kwargs={"pk": self.myuser.pk})
        )
        self.assertTemplateUsed(response, "auth/user_detail.html")
        self.assertEqual(response.status_code, 200, response)

        # The other user can also be accessed
        response = self.client.get(
            reverse("user-detail", kwargs={"pk": self.foreignuser.pk})
        )
        self.assertEqual(response.status_code, 200)

    def test_otherusers_edit(self):
        url = reverse("user-update", kwargs={"pk": self.myuser.pk})
        response = self.client.get(url)
        self.assertTemplateUsed(response, "auth/user_form.html")
        self.assertEqual(response.status_code, 200, response)

        # Our orga cannot be edited away from my cantons
        initial = self.getprofileinitial(self.myuser)
        initial["address_no"] = 24

        response = self.client.post(url, initial)
        # Code 302 because update succeeded
        self.assertEqual(response.status_code, 302, url)
        # Check update succeeded
        newuser = get_user_model().objects.get(pk=self.myuser.pk)
        self.assertEqual(newuser.profile.address_no, "24")

        # Test modifying the affiliation canton _away_
        initial["affiliation_canton"] = self.foreignuser.profile.affiliation_canton
        # And modifying settings only accessible to Bureau
        initial["employee_code"] = "rich"
        initial["comments"] = "Nasty comment"

        response = self.client.post(url, initial)
        # Code 200 because update failed
        self.assertEqual(response.status_code, 200, url)
        # Check update failed
        newuser = get_user_model().objects.get(pk=self.myuser.pk)
        self.assertEqual(
            newuser.profile.affiliation_canton, self.myuser.profile.affiliation_canton
        )
        self.assertEqual(newuser.profile.employee_code, "poor")
        self.assertEqual(newuser.profile.comments, "")

        # The other user cannot be accessed
        response = self.client.get(
            reverse("user-update", kwargs={"pk": self.foreignuser.pk})
        )
        self.assertEqual(response.status_code, 403)

    def test_cannot_update_cresus_number(self):
        url = reverse("user-update", kwargs={"pk": self.myuser.pk})
        initial = self.getprofileinitial(self.myuser)
        initial["employee_code"] = "rich"

        self.client.post(url, initial)

        self.myuser.profile.refresh_from_db()
        self.assertEqual(self.myuser.profile.employee_code, "poor")

    def test_autocompletes(self):
        for al in ["AllPersons"]:
            url = reverse("user-%s-ac" % al)
            response = self.client.get(url)
            self.assertEqual(response.status_code, 200, url)
            # Check that we find everyone, including users of other cantons
            entries = re.findall(r'"id": "(\d+)"', str(response.content))
            expected_entries = set(str(u.id) for u in User.objects.all())
            self.assertEqual(set(entries), expected_entries)


class SuperUserTest(ProfileTestCase):
    def setUp(self):
        super(SuperUserTest, self).setUp()
        self.client = SuperUserAuthClient()



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("challenge", "0009_qualification_actor"),
    ]

    operations = [
        migrations.AddField(
            model_name="qualification",
            name="comments",
            field=models.TextField(verbose_name="Remarques", blank=True),
        ),
        migrations.AddField(
            model_name="qualification",
            name="route",
            field=models.CharField(
                max_length=1,
                verbose_name="Parcours",
                choices=[("A", "A"), ("B", "B"), ("C", "C"), ("D", "D")],
                blank=True,
            ),
        ),
        migrations.AddField(
            model_name="session",
            name="comments",
            field=models.TextField(verbose_name="Remarques", blank=True),
        ),
    ]



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("challenge", "0022_auto_20150930_1608"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="session",
            name="chosen_staff",
        ),
        migrations.AddField(
            model_name="helpersessionavailability",
            name="chosen",
            field=models.BooleanField(
                verbose_name="Sélectionné pour la session", default=False
            ),
        ),
    ]



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("challenge", "0020_auto_20150917_1710"),
    ]

    operations = [
        migrations.AlterField(
            model_name="helpersessionavailability",
            name="session",
            field=models.ForeignKey(
                verbose_name="Session",
                related_name="availability_statuses",
                to="challenge.Session",
                on_delete=models.CASCADE,
            ),
        ),
    ]



# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2016, 2020 Didier Raboud <me+defivelo@odyx.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import datetime

from django.test import TestCase
from django.urls import reverse

from defivelo.tests.utils import (
    CollaboratorAuthClient,
    StateManagerAuthClient,
)

from .factories import SeasonFactory, SessionFactory


class SessionCloneViewTest(TestCase):
    def setUp(self):
        self.client = StateManagerAuthClient()
        self.season = SeasonFactory(
            cantons=["VD"], year=2019, month_start=1, n_months=3
        )
        self.session = SessionFactory(
            orga__address_canton="VD",
            day=datetime.date(2019, 2, 11),
            begin=datetime.time(9, 30),
        )
        super().setUp()

    def test_get_session_clone_view(self):
        url = reverse(
            "session-clone", kwargs={"seasonpk": self.season.pk, "pk": self.session.pk}
        )

        post_url = reverse("session-create", kwargs={"seasonpk": self.season.pk})

        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

        context_session = response.context["session"]
        self.assertIsNone(context_session.pk)
        self.assertIsNone(context_session.day)
        self.assertIsNone(context_session.begin)

        self.assertContains(response, 'action="{}"'.format(post_url))

    def test_get_form_kwargs_clears_clone_fields(self):
        url = reverse(
            "session-clone", kwargs={"seasonpk": self.season.pk, "pk": self.session.pk}
        )

        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

        instance = response.context["form"].instance
        self.assertIsNone(instance.pk)
        self.assertIsNone(instance.day)
        self.assertIsNone(instance.begin)

    def test_post_not_allowed_raises_error(self):
        url = reverse(
            "session-clone", kwargs={"seasonpk": self.season.pk, "pk": self.session.pk}
        )

        response = self.client.post(url, {})
        self.assertEqual(response.status_code, 403)  # PermissionDenied

    def test_clone_payload_can_create(self):
        url_clone = reverse(
            "session-clone", kwargs={"seasonpk": self.season.pk, "pk": self.session.pk}
        )
        response = self.client.get(url_clone)
        self.assertEqual(response.status_code, 200)

        session = response.context["form"].instance

        data = {
            "orga": session.orga.pk,
            "place": session.place,
            "address_street": session.address_street,
            "address_no": session.address_no,
            "address_zip": session.address_zip,
            "address_city": session.address_city,
            "address_canton": session.address_canton,
            "fallback_plan": "A",
            "visible": "on",
            "superleader": "",
            "bikes_concept": "Keine",
            "bikes_phone": "+41761121232",
            "apples": "im Cargo",
            "helpers_time": "09:10",
            "helpers_place": session.helpers_place,
            "comments": "",
            "day": "2019-03-11",
            "begin": "09:30",
        }

        url = reverse("session-create", kwargs={"seasonpk": self.season.pk})
        response = self.client.post(url, data)
        self.assertEqual(response.status_code, 302)
        self.assertIn(
            "/fr/season/{}".format(self.season.pk),
            response.headers.get("location"),
            "Invalid Redirect url",
        )

    def test_clone_view_requires_challenge_session_crud_permission(self):
        client_without_permission = CollaboratorAuthClient()
        url = reverse(
            "session-clone", kwargs={"seasonpk": self.season.pk, "pk": self.session.pk}
        )

        response = client_without_permission.get(url)
        self.assertEqual(response.status_code, 403)



# Generated by Django 2.2.8 on 2019-12-12 13:54

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("user", "0047_userprofile_marital_status"),
    ]

    operations = [
        migrations.AlterField(
            model_name="usermanagedstate",
            name="user",
            field=models.ForeignKey(
                limit_choices_to={"is_active": True},
                on_delete=django.db.models.deletion.CASCADE,
                related_name="managedstates",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AlterField(
            model_name="userprofile",
            name="actor_for",
            field=models.ManyToManyField(
                blank=True,
                limit_choices_to={"category": "C"},
                related_name="actor_for",
                to="challenge.QualificationActivity",
                verbose_name="Intervenant",
            ),
        ),
    ]



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("challenge", "0004_auto_20150831_1529"),
    ]

    operations = [
        migrations.CreateModel(
            name="Qualification",
            fields=[
                (
                    "id",
                    models.AutoField(
                        verbose_name="ID",
                        primary_key=True,
                        serialize=False,
                        auto_created=True,
                    ),
                ),
                ("created_on", models.DateTimeField(auto_now_add=True)),
                ("name", models.CharField(verbose_name="Nom", max_length=255)),
            ],
            options={
                "ordering": ["session", "created_on", "name"],
                "verbose_name": "Qualification",
                "verbose_name_plural": "Qualifications",
            },
        ),
        migrations.AlterModelOptions(
            name="session",
            options={
                "ordering": ["day", "timeslot__begin", "organization__name"],
                "verbose_name": "Session",
                "verbose_name_plural": "Sessions",
            },
        ),
        migrations.AlterField(
            model_name="session",
            name="organization",
            field=models.ForeignKey(
                blank=True,
                null=True,
                related_name="sessions",
                verbose_name="Établissement",
                to="orga.Organization",
                on_delete=models.CASCADE,
            ),
        ),
        migrations.AlterField(
            model_name="session",
            name="timeslot",
            field=models.ForeignKey(
                blank=True,
                null=True,
                related_name="sessions",
                verbose_name="Horaire",
                to="challenge.SessionTimeSlot",
                on_delete=models.CASCADE,
            ),
        ),
        migrations.AddField(
            model_name="qualification",
            name="session",
            field=models.ForeignKey(
                to="challenge.Session",
                related_name="qualifications",
                on_delete=models.CASCADE,
            ),
        ),
    ]



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Organization",
            fields=[
                (
                    "id",
                    models.AutoField(
                        primary_key=True,
                        verbose_name="ID",
                        serialize=False,
                        auto_created=True,
                    ),
                ),
                ("created_on", models.DateTimeField(auto_now_add=True)),
                ("name", models.CharField(max_length=255, verbose_name="Nom")),
                ("address_street", models.CharField(max_length=255, blank=True)),
                ("address_no", models.CharField(max_length=8, blank=True)),
                ("address_zip", models.CharField(max_length=4, blank=True)),
                ("address_city", models.CharField(max_length=64, blank=True)),
                ("address_canton", models.CharField(max_length=2, blank=True)),
            ],
            options={
                "verbose_name_plural": "Établissements",
                "verbose_name": "Établissement",
                "ordering": ["name"],
            },
        ),
    ]



from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("challenge", "0029_auto_20161010_1518"),
    ]

    operations = [
        migrations.AlterField(
            model_name="session",
            name="organization",
            field=models.ForeignKey(
                related_name="sessions",
                to="orga.Organization",
                verbose_name="Établissement",
                default=5,
                on_delete=models.CASCADE,
            ),
            preserve_default=False,
        ),
    ]



# Generated by Django 2.2.9 on 2020-03-05 15:31

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("challenge", "0058_invoice_status_label"),
    ]

    operations = [
        migrations.AlterField(
            model_name="invoice",
            name="season",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name="invoices",
                to="challenge.Season",
                verbose_name="Saison",
            ),
        ),
    ]



from django.contrib import admin
from django.urls import re_path

from . import views

urlpatterns = [
    re_path(
        r"^outbox/$",
        admin.site.admin_view(views.outbox_list_view),
        name="email-outbox-list",
    ),
    re_path(
        r"^outbox/(?P<idx>\d+)/$",
        admin.site.admin_view(views.outbox_detail_view),
        name="email-outbox-detail",
    ),
    re_path(
        r"^outbox/(?P<idx>\d+)/body/$",
        admin.site.admin_view(views.outbox_body_view),
        name="email-outbox-body",
    ),
    re_path(
        r"^outbox/(?P<idx>\d+)/attachments/(?P<aidx>\d+)/$",
        admin.site.admin_view(views.outbox_attachment_view),
        name="email-outbox-attachment",
    ),
]



# Generated by Django 1.11.4 on 2017-08-17 12:24
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("orga", "0003_auto_20161014_1029"),
    ]

    operations = [
        migrations.AddField(
            model_name="organization",
            name="coordinator_phone",
            field=models.CharField(blank=True, max_length=13, verbose_name="Téléphone"),
        ),
        migrations.AddField(
            model_name="organization",
            name="website",
            field=models.URLField(blank=True, verbose_name="Site web"),
        ),
    ]



# Generated by Django 4.2.6 on 2025-11-13 13:46

from django.conf import settings
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('challenge', '0085_historicalseason_availability_reminder_sent_at'),
    ]

    operations = [
        migrations.AlterField(
            model_name='annualstatesetting',
            name='cost_per_participant',
            field=models.DecimalField(decimal_places=2, default=0, help_text='CHF', max_digits=8, validators=[django.core.validators.MinValueValidator(0)], verbose_name='Coût par participant·e·s'),
        ),
        migrations.AlterField(
            model_name='helpersessionavailability',
            name='chosen_as',
            field=models.PositiveSmallIntegerField(choices=[(0, 'Pas choisi'), (1, 'Choisi'), (2, 'Comme intervenant·e'), (5, 'Moniteur·trice de secours'), (3, 'Moniteur·trice 1'), (4, 'Moniteur·trice 2')], default=0, verbose_name='Sélectionné pour la session comme'),
        ),
        migrations.AlterField(
            model_name='historicalannualstatesetting',
            name='cost_per_participant',
            field=models.DecimalField(decimal_places=2, default=0, help_text='CHF', max_digits=8, validators=[django.core.validators.MinValueValidator(0)], verbose_name='Coût par participant·e·s'),
        ),
        migrations.AlterField(
            model_name='historicalhelpersessionavailability',
            name='chosen_as',
            field=models.PositiveSmallIntegerField(choices=[(0, 'Pas choisi'), (1, 'Choisi'), (2, 'Comme intervenant·e'), (5, 'Moniteur·trice de secours'), (3, 'Moniteur·trice 1'), (4, 'Moniteur·trice 2')], default=0, verbose_name='Sélectionné pour la session comme'),
        ),
        migrations.AlterField(
            model_name='historicalqualification',
            name='actor',
            field=models.ForeignKey(blank=True, db_constraint=False, limit_choices_to={'profile__actor_for__isnull': False}, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to=settings.AUTH_USER_MODEL, verbose_name='Intervenant·e'),
        ),
        migrations.AlterField(
            model_name='historicalqualification',
            name='class_teacher_fullname',
            field=models.CharField(blank=True, max_length=512, verbose_name='Enseignant·e'),
        ),
        migrations.AlterField(
            model_name='historicalqualification',
            name='class_teacher_natel',
            field=models.CharField(blank=True, max_length=13, verbose_name='Natel enseignant·e'),
        ),
        migrations.AlterField(
            model_name='historicalqualification',
            name='leader',
            field=models.ForeignKey(blank=True, db_constraint=False, limit_choices_to=models.Q(('profile__formation', 'M2')), null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to=settings.AUTH_USER_MODEL, verbose_name='Moniteur·trice 2'),
        ),
        migrations.AlterField(
            model_name='historicalqualification',
            name='n_helpers',
            field=models.IntegerField(choices=[(1, '1'), (2, '2'), (3, '3')], default=3, verbose_name='Nombre de moniteur·trice·s'),
        ),
        migrations.AlterField(
            model_name='historicalseason',
            name='leader',
            field=models.ForeignKey(blank=True, db_constraint=False, limit_choices_to={'managedstates__isnull': False}, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to=settings.AUTH_USER_MODEL, verbose_name='Chargé·e de projet'),
        ),
        migrations.AlterField(
            model_name='historicalseason',
            name='state',
            field=models.PositiveSmallIntegerField(choices=[(1, 'Planification (invisible)'), (2, 'Annoncé (rentrée des disponibilités)'), (3, 'En cours (corrections que par les chargé·e·s de projet)'), (4, 'Terminé'), (5, 'Archivé·e')], default=1, verbose_name='État'),
        ),
        migrations.AlterField(
            model_name='historicalsession',
            name='helpers_place',
            field=models.CharField(blank=True, max_length=512, verbose_name='Lieu rendez-vous moniteur·trice·s'),
        ),
        migrations.AlterField(
            model_name='historicalsession',
            name='helpers_time',
            field=models.TimeField(blank=True, null=True, verbose_name='Heure de rendez-vous des moniteur·trice·s'),
        ),
        migrations.AlterField(
            model_name='historicalsession',
            name='superleader',
            field=models.ForeignKey(blank=True, db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to=settings.AUTH_USER_MODEL, verbose_name='Moniteur·trice + / Photographe'),
        ),
        migrations.AlterField(
            model_name='historicalsession',
            name='visible',
            field=models.BooleanField(default=False, verbose_name='Visible pour les moniteur·trice·s'),
        ),
        migrations.AlterField(
            model_name='qualification',
            name='actor',
            field=models.ForeignKey(blank=True, limit_choices_to={'profile__actor_for__isnull': False}, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='qualifs_actor', to=settings.AUTH_USER_MODEL, verbose_name='Intervenant·e'),
        ),
        migrations.AlterField(
            model_name='qualification',
            name='class_teacher_fullname',
            field=models.CharField(blank=True, max_length=512, verbose_name='Enseignant·e'),
        ),
        migrations.AlterField(
            model_name='qualification',
            name='class_teacher_natel',
            field=models.CharField(blank=True, max_length=13, verbose_name='Natel enseignant·e'),
        ),
        migrations.AlterField(
            model_name='qualification',
            name='helpers',
            field=models.ManyToManyField(blank=True, limit_choices_to=models.Q(('profile__formation__in', ['M1', 'M2'])), related_name='qualifs_mon1', to=settings.AUTH_USER_MODEL, verbose_name='Moniteur·trice·s 1'),
        ),
        migrations.AlterField(
            model_name='qualification',
            name='leader',
            field=models.ForeignKey(blank=True, limit_choices_to=models.Q(('profile__formation', 'M2')), null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='qualifs_mon2', to=settings.AUTH_USER_MODEL, verbose_name='Moniteur·trice 2'),
        ),
        migrations.AlterField(
            model_name='qualification',
            name='n_helpers',
            field=models.IntegerField(choices=[(1, '1'), (2, '2'), (3, '3')], default=3, verbose_name='Nombre de moniteur·trice·s'),
        ),
        migrations.AlterField(
            model_name='season',
            name='leader',
            field=models.ForeignKey(limit_choices_to={'managedstates__isnull': False}, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='Chargé·e de projet'),
        ),
        migrations.AlterField(
            model_name='season',
            name='state',
            field=models.PositiveSmallIntegerField(choices=[(1, 'Planification (invisible)'), (2, 'Annoncé (rentrée des disponibilités)'), (3, 'En cours (corrections que par les chargé·e·s de projet)'), (4, 'Terminé'), (5, 'Archivé·e')], default=1, verbose_name='État'),
        ),
        migrations.AlterField(
            model_name='session',
            name='helpers_place',
            field=models.CharField(blank=True, max_length=512, verbose_name='Lieu rendez-vous moniteur·trice·s'),
        ),
        migrations.AlterField(
            model_name='session',
            name='helpers_time',
            field=models.TimeField(blank=True, null=True, verbose_name='Heure de rendez-vous des moniteur·trice·s'),
        ),
        migrations.AlterField(
            model_name='session',
            name='superleader',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='sess_monplus', to=settings.AUTH_USER_MODEL, verbose_name='Moniteur·trice + / Photographe'),
        ),
        migrations.AlterField(
            model_name='session',
            name='visible',
            field=models.BooleanField(default=False, verbose_name='Visible pour les moniteur·trice·s'),
        ),
        migrations.AlterField(
            model_name='historicalseason',
            name='state',
            field=models.PositiveSmallIntegerField(
                choices=[(1, 'Planification (invisible)'),
                         (2, 'Annoncé·e (rentrée des disponibilités)'),
                         (3, 'En cours (corrections que par les chargé·e·s de projet)'),
                         (4, 'Terminé·e'), (5, 'Archivé·e')], default=1,
                verbose_name='État'),
        ),
        migrations.AlterField(
            model_name='season',
            name='state',
            field=models.PositiveSmallIntegerField(
                choices=[(1, 'Planification (invisible)'),
                         (2, 'Annoncé·e (rentrée des disponibilités)'),
                         (3, 'En cours (corrections que par les chargé·e·s de projet)'),
                         (4, 'Terminé·e'), (5, 'Archivé·e')], default=1,
                verbose_name='État'),
        ),
        migrations.AlterField(
            model_name='historicalseason',
            name='state',
            field=models.PositiveSmallIntegerField(choices=[(1, 'Planification (invisible)'), (2, 'Annoncé (rentrée des disponibilités)'), (3, 'En cours (corrections que par chargé·e·s de projet)'), (4, 'Terminé'), (5, 'Archivé')], default=1, verbose_name='État'),
        ),
        migrations.AlterField(
            model_name='season',
            name='state',
            field=models.PositiveSmallIntegerField(choices=[(1, 'Planification (invisible)'), (2, 'Annoncé (rentrée des disponibilités)'), (3, 'En cours (corrections que par chargé·e·s de projet)'), (4, 'Terminé'), (5, 'Archivé')], default=1, verbose_name='État'),
        ),
    ]



# defivelo-intranet -- Outil métier pour la gestion du Défi Vélo
# Copyright (C) 2020 Didier 'OdyX' Raboud <didier.raboud@liip.ch>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from django.contrib.messages.views import SuccessMessageMixin
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _
from django.views.generic import ListView
from django.views.generic.edit import CreateView, UpdateView

from rolepermissions.mixins import HasPermissionsMixin

from apps.common import DV_STATES
from defivelo.roles import has_permission
from defivelo.views import MenuView

from ..forms import AnnualStateSettingForm
from ..models import AnnualStateSetting


class SettingsMixin(HasPermissionsMixin):
    model = AnnualStateSetting
    required_permission = "settings_crud"

    def dispatch(self, request, *args, **kwargs):
        self.year = kwargs.pop("year")
        self.cantons = (
            DV_STATES
            if has_permission(request.user, "cantons_all")
            else self.request.user.managedstates.all().values_list("canton", flat=True)
        )
        return super().dispatch(request, *args, **kwargs)

    def get_queryset(self):
        return super().get_queryset().filter(year=self.year, canton__in=self.cantons)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        # Add our menu_category context
        context["menu_category"] = "settings"
        context["year"] = self.year
        return context

    def get_form_kwargs(self):
        form_kwargs = super().get_form_kwargs()
        form_kwargs["year"] = self.year
        form_kwargs["cantons"] = self.cantons
        return form_kwargs

    def get_success_url(self):
        return reverse_lazy(
            "annualstatesettings-list", kwargs={"year": self.object.year}
        )


class AnnualStateSettingsListView(SettingsMixin, MenuView, ListView):
    context_object_name = "settings"
    ordering = ["canton"]


class AnnualStateSettingMixin(SettingsMixin, SuccessMessageMixin, MenuView):
    context_object_name = "setting"
    form_class = AnnualStateSettingForm


class AnnualStateSettingCreateView(AnnualStateSettingMixin, CreateView):
    success_message = _("Configuration cantonale par année créée")


class AnnualStateSettingUpdateView(AnnualStateSettingMixin, UpdateView):
    success_message = _("Configuration cantonale par année mise à jour")



from __future__ import unicode_literals

import localflavor.generic.models
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("user", "0013_auto_20151015_1824"),
    ]

    operations = [
        migrations.AlterField(
            model_name="userprofile",
            name="iban",
            field=localflavor.generic.models.IBANField(
                False,
                (
                    "AT",
                    "BE",
                    "BG",
                    "CH",
                    "CY",
                    "CZ",
                    "DE",
                    "DK",
                    "EE",
                    "ES",
                    "FI",
                    "FR",
                    "GB",
                    "GI",
                    "GR",
                    "HR",
                    "HU",
                    "IE",
                    "IS",
                    "IT",
                    "LI",
                    "LT",
                    "LU",
                    "LV",
                    "MC",
                    "MT",
                    "NL",
                    "NO",
                    "PL",
                    "PT",
                    "RO",
                    "SE",
                    "SI",
                    "SK",
                    "SM",
                ),
                blank=True,
            ),
        ),
    ]



# Generated by Django 2.2.16 on 2020-10-28 15:58

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("challenge", "0072_season_to_month_in_names"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalseason",
            name="state",
            field=models.PositiveSmallIntegerField(
                choices=[
                    (1, "Planification (invisible)"),
                    (2, "Annoncé (rentrée des disponibilités)"),
                    (3, "En cours (corrections que par chargé·e·s de projet)"),
                    (4, "Terminé"),
                    (5, "Archivé"),
                ],
                default=1,
                verbose_name="État",
            ),
        ),
        migrations.AlterField(
            model_name="season",
            name="state",
            field=models.PositiveSmallIntegerField(
                choices=[
                    (1, "Planification (invisible)"),
                    (2, "Annoncé (rentrée des disponibilités)"),
                    (3, "En cours (corrections que par chargé·e·s de projet)"),
                    (4, "Terminé"),
                    (5, "Archivé"),
                ],
                default=1,
                verbose_name="État",
            ),
        ),
    ]



# Generated by Django 2.2.9 on 2020-04-09 00:49

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("salary", "0006_data_update_timesheets"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="timesheet",
            name="time_actor",
        ),
        migrations.AddField(
            model_name="timesheet",
            name="actor_count",
            field=models.IntegerField(default=0, verbose_name="Intervention(s)"),
        ),
    ]



# Generated by Django 1.11.4 on 2017-08-25 11:49
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("challenge", "0039_auto_20170824_1222"),
    ]

    operations = [
        migrations.AlterField(
            model_name="session",
            name="place",
            field=models.CharField(
                blank=True, max_length=512, verbose_name="Lieu de la Qualif'"
            ),
        ),
    ]



# Generated by Django 2.2.9 on 2020-04-08 12:10

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("salary", "0004_auto_20200407_0949"),
    ]

    operations = [
        migrations.AddField(
            model_name="timesheet",
            name="leader_count",
            field=models.IntegerField(
                default=0, verbose_name="Participation(s) comme Moniteur·trice 2"
            ),
        ),
    ]



# Generated by Django 1.11.4 on 2017-08-24 09:09
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("user", "0039_rename_back"),
    ]

    operations = [
        migrations.AlterField(
            model_name="userprofile",
            name="actor_for",
            field=models.ManyToManyField(
                blank=True,
                related_name="actor_for",
                to="challenge.QualificationActivity",
                verbose_name="Intervenant",
            ),
        ),
    ]



# Generated by Django 2.2.12 on 2020-10-26 14:23

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("challenge", "0065_auto_20200512_1638"),
    ]

    operations = [
        migrations.AlterField(
            model_name="invoiceline",
            name="session",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="challenge.Session",
            ),
        ),
    ]
